function q = curve_to_q(p,varargin)

if isempty(varargin)
    t = 0;
else 
    t = varargin{1};
    if length(t) ~= size(p,2)
        error('The length of t has to be the same as the number of columns in p!');
    end
end

[dim,N] = size(p);

dp = zeros(dim,N);
for k = 1:dim
    dp(k,:) = num_deriv(p(k,:),t); % Numerical derivative of p.
end

% if p is closed curve, first and last nodes are equal: use centered diff.
if norm(p(:,1)-p(:,N)) < 100*eps
    if isscalar(t) % The uniform case.
        h = 1 / (N-1);
        dp(:,1) = (p(:,2) - p(:,N-1)) / (2*h);
    else % The nonuniform case.
        h = [ t(N)-t(N-1) t(2)-t(1) ];
        three_nodes = [p(:,N-1) p(:,1) p(:,2)];  % Note: p(:,1) == p(:,N)
        dp(:,1) = centered_finite_difference(three_nodes,h);
    end
    dp(:,N) = dp(:,1);
end

norm_dp = sqrt( sum(dp.^2,1) );

if dim == 1
    q = sign(dp) .* sqrt(norm_dp);
else % dim = 2,3,...
    q = zeros(size(p));
    for k = 1:dim
        q(k,:) = dp(k,:) ./ sqrt(norm_dp);
    end
end

q = q / sqrt(InnerProd_Q(q,q));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function df = num_deriv(f,t)

[dim,N] = size(f);
if isscalar(t) % The uniform case.
    h = 1 / (N-1);
    df = gradient(f,h);
else % Nonuniform case: grid pts t provided, lead to nonuniform spacing h.
    df = zeros(dim,N);
    h = t(2:N) - t(1:N-1);
    df(:,2:N-1) = centered_finite_difference(f,h);
    df(:,1) = (f(:,2) - f(:,1)) / h(1);
    df(:,N) = (f(:,N) - f(:,N-1)) / h(N-1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function df = centered_finite_difference(f,h)

N = size(f,2);
w1 = h(1:N-2) ./ h(2:N-1);   w2 = 1./w1;
df = (w1.*(f(:,3:N) - f(:,2:N-1)) + w2.*(f(:,2:N-1) - f(:,1:N-2))) ...
    ./ (h(1:N-2)+h(2:N-1));
