#!/usr/bin/env python3

import numpy as np
import pylab as plt
import statsmodels.api as sm
from scipy.interpolate import splrep, splev
import dp_pyth_wndstrp

def inner_prod_q_1d(q1,q2):
    # Approximation of L2 inner product
    T = q1.shape
    val = np.sum(q1*q2) / T
    return val

def curve_to_q_1d(f,t,N):
    dp = np.zeros(N)
    h = t[1:N] - t[0:N-1]
    dp[0] = (f[1]-f[0])/h[0]
    dp[N-1] = (f[N-1]-f[N-2])/h[N-2]
    for i in range(1,N-1):
      dp[i] = ((h[i-1]/h[i])*(f[i+1]-f[i])+(h[i]/h[i-1])*(f[i]-f[i-1]))/(h[i-1]+h[i])

    q = np.zeros(N)
    for i in range(N):
      q[i] = np.sign(dp[i]) * np.sqrt(np.abs(dp[i]))

    q = q / np.sqrt(inner_prod_q_1d(q,q))
    return q

f1 = np.genfromtxt('function1.txt')
#savetxt('func1.txt',f1, fmt='%.6f')
f2 = np.genfromtxt('function2.txt')
#savetxt('func2.txt',f2, fmt='%.6f')
p1 = f1.T
p2 = f2.T

n1, N1 = p1.shape
n2, N2 = p2.shape
print('n1 =',n1,'N1 =',N1)
print('n2 =',n2,'N2 =',N2)

if(n1 != 2 or n2 !=2):
   raise Exception("Incorrect first parameter for p1, p2")

r1 = p1[0,:]
b1 = p1[1,:]
r2 = p2[0,:]
b2 = p2[1,:]

#print(r1.min())
#print(r1.max())
#print(r2.min())
#print(r2.max())

dt = r1.max() - r1.min()
t1 = (r1 - r1.min())/dt
t1[0] = 0
t1[N1-1] = 1
#print('t1[0] =',t1[0],' t1[N1-1] = ',t1[N1-1])

dt = r2.max() - r2.min()
t2 = (r2 - r2.min())/dt
t2[0] = 0
t2[N2-1] = 1
#print('t2[0] =',t2[0],' t2[N2-1] = ',t2[N2-1])

f1=b1
f2=b2

imath=0
if(imath):
  lowess1 = sm.nonparametric.lowess(b1, t1, frac=0.0025)
  f1 = lowess1[:,1]
  lowess2 = sm.nonparametric.lowess(b2, t2, frac=0.0025)
  f2 = lowess2[:,1]

#print(np.shape(t1))
#print(np.shape(b1))
#print(np.shape(lowess1))

del1 = np.zeros(N1)
for j in range(1,N1):
    del1[j] = np.abs(f1[j]-f1[j-1])

del2 = np.zeros(N2)
for j in range(1,N2):
    del2[j] = np.abs(f2[j]-f2[j-1])

#print(f1[0:10])
#print(del1[0:10])
#print(f2[0:10])
#print(del2[0:10])

#print(np.shape(del1))
#print(np.shape(del2))

len1 = np.sum(del1)
len2 = np.sum(del2)
leng = len1
if leng < len2:
   leng = len2

#print('len1 =',len1,' len2 =',len2,' leng =',leng)

f1 = f1/leng
f2 = f2/leng

q12 = curve_to_q_1d(f1,t1,N1)
q22 = curve_to_q_1d(f2,t2,N2)

#print(q[0:10])
#print(q[N1-10:N1])

#plt.figure()
#plt.plot(t1,q12,'b')

#plt.figure()
#plt.plot(t2,q22,'r')


plt.figure()
plt.plot(t1,b1,'b')
plt.figure()
plt.plot(t1,f1,'b')


plt.figure()
plt.plot(t2,b2,'r')
plt.figure()
plt.plot(t2,f2,'r')

q1 = np.zeros((2,N1))
q1[1,:] = q12
q2 = np.zeros((2,N2))
q2[1,:] = q22


layrs = 12
lstrp = 150
iopt = 1
mnrgy = -1000.0;
pfly = 0;
pflp = 0;


lgrid = N1
jgrid = N2
#print(np.shape(q1))
#print(lgrid)
#print(np.shape(q2))
#print(jgrid)

print(' ')
print('Executing DP computation ...')
print(' ')

(new_gamma, pfly, pflp, mnrgy, ilowb, ihigh, lbnd, hbnd) = \
       dp_pyth_wndstrp.dp_pyth_wndstrp(q2, q1, t2, t1, \
       layrs, lstrp, iopt, jgrid, lgrid)
# Note here the order is q2, q1, so must have t2, t1, and jgrid, lgrid, in that order.
# If the order is q1, q2, then must have t1, t2, and lgrid, jgrid, in that order.

print(' ')
print('DP computation completed.') 
print(' ')

print('ilowb =',ilowb)
print('ihigh =',ihigh)

print('mnrgy =',mnrgy)
print('pfly =',pfly)
print('pflp =',pflp)

plt.figure()
plt.plot(t2,new_gamma,'b')

tck = splrep(t1,b1,k=3)
b1_gamma = splev(new_gamma,tck)

plt.figure()
plt.plot(t1,b1,'b',t2,b2,'r')

plt.figure()
plt.plot(t2,b1_gamma,'b',t2,b2,'r')

tck = splrep(t1,f1,k=3)
f1_gamma = splev(new_gamma,tck)

plt.figure()
plt.plot(t1,f1,'b',t2,f2,'r')

plt.figure()
plt.plot(t2,f1_gamma,'b',t2,f2,'r')

plt.figure()
plt.plot(lbnd[0,0:ilowb],lbnd[1,0:ilowb],'g.',hbnd[0,0:ihigh],hbnd[1,0:ihigh],'g.')

plt.figure()
plt.plot(lbnd[0,0:ilowb],lbnd[1,0:ilowb],'g.',hbnd[0,0:ihigh],hbnd[1,0:ihigh],'g.',t2,new_gamma,'b-')

plt.show()

